// compensation of uneven print
print_expansion = 0.1;

holder_length = 30;
holder_inner_diameter = [29, 32]; // first size must be less or equal than second
holder_thickness = 1.5;

joint_size = [16, 12, 18];
joint_slot_width = 4;
joint_hole_diameter = 7;
joint_cogs_count=30;
joint_cogs_height=0.5;
joint_cogs_offset=1;

// technical overlap between joined or deducted details to ensure volumes are combined in one and deducted without thin planes at the edges
overlap=0.1;
pi=3.14;

// this is the angle for whole detail to rotate to make holder semi-circle perpendicular to XY plane
ratio = holder_inner_diameter[1]/holder_inner_diameter[0];
tilt_angle = atan(((holder_inner_diameter[0]/2+holder_thickness)*ratio-(holder_inner_diameter[0]/2+holder_thickness))/holder_length);;

module holder() {
	length = holder_length;
	inner_radius = [holder_inner_diameter[0]/2, holder_inner_diameter[1]/2];
	thickness = holder_thickness;
	
	outer_radius = [inner_radius[0]+thickness, (inner_radius[0]+thickness)*ratio];
	
    // rotate holder to make it horizontal before attaching to the joint
	rotate( [-tilt_angle,0,0] )
    translate( [0,0,-outer_radius[0]*(1+ratio)/2] )
    rotate( [-90,0,0] )
	rotate( [0,0,45] )
	linear_extrude( height=length, slices=ceil(length), scale=ratio, center=true )
	union() {
		difference() {
			circle( $fn=2*outer_radius[0]*pi*3, r=outer_radius[0] );
			circle( $fn=2*inner_radius[0]*pi*3, r=inner_radius[0] );
			square( outer_radius[0]+overlap );
		}
		
		translate( [inner_radius[0]+thickness/2, 0] )
		circle( $fn=12, d=thickness );

		translate( [0, inner_radius[0]+thickness/2] )
		circle( $fn=12, d=thickness );
	}
}

// small cogs to make a traction in joint
module cogs() {
    count = joint_cogs_count;
    angle=360/count;

	offset = joint_cogs_offset;
	outer_radius = joint_size[0]/2-offset;
	inner_radius = joint_hole_diameter/2+offset;

    height = joint_cogs_height;
	length = outer_radius*cos(angle/2)-inner_radius;
    
	outer_width=outer_radius*sin(angle/2)*2;
	inner_width=inner_radius*tan(angle/2)*2;
	
    for( number = [1 : count] ) {
        rotate( [0,number*angle,0] )
		translate( [0,0,-outer_radius*cos(angle/2)] )
		linear_extrude( height=length, slices=ceil(length), scale=[(inner_width/outer_width),1] )
		polygon( points=[[-outer_width/2,0], [0,height], [outer_width/2,0], [outer_width/2,-overlap], [-outer_width/2,-overlap]] );
    };
}

// joint arm
module joint () {
    width = joint_size[0];
    depth = joint_slot_width;
    height = joint_size[2];

	holder_min_radius = holder_inner_diameter[0]/2+holder_thickness;
	insert_angle = asin((depth/2)/holder_min_radius);
	insert_length = holder_min_radius*(1-cos(insert_angle))+overlap;

	translate( [0, 0, -insert_length/2] )
	cube( [depth, width, insert_length], center=true );
	
	union() {
		// joint without cogs
		difference() {
			// joint full body
			union() {
				// cubical part of the joint
				translate( [-depth/2, -width/2, -overlap] )
				cube( [depth, width, height-width/2+overlap] );
				
				// radial part of the joint
				translate( [0,0,height-width/2] )
				rotate( [0,90,0] ) 
				cylinder( $fn=width*pi*3, depth, d=width, center=true );
			}
			
			// mount hole
			translate( [0,0,height-width/2] )
			rotate( [0,90,0] ) 
			cylinder( $fn=joint_hole_diameter*pi*3, depth+overlap*2, d=joint_hole_diameter, center=true );

			// cogs
			rotate( [0,0,90] )
			translate( [0,-depth/2,height-width/2] )
			cogs();

			rotate( [0,0,-90] )
			translate( [0,-depth/2,height-width/2] )
			cogs();
		}
	}
	
}

// make wide side of the holder laying flat at XY plane
translate ( [0,0,holder_length/2] )
rotate( [-90+tilt_angle,0,0] )
union() {
	holder();

	joint();
}